#! /usr/bin/env python3
"""
## Command ``define``

Searches for the definition of the word given in parameter (french). See `define --help` for more details.

### Example

```
$ # searches for the definition of 'gentil'
$ define gentil
```
"""

import click
from bs4 import BeautifulSoup
import requests
import re
from urllib.parse import quote

@click.command()
@click.argument('word')
def main(word):
    """Retrieve french definition."""
    url_larousse = 'http://www.larousse.fr/dictionnaires/rechercher?q={}&l=francais&culture='
    url = url_larousse.format(quote(word))
    r = requests.get(url)
    soup = BeautifulSoup(r.content, 'html.parser')
    defs = soup.find_all('li',attrs={"class":u"DivisionDefinition"})
    if len(defs)<1:
        print("Nothing found...")
        return
    for def_ in defs:
        print('--'+def_.text)

if __name__ == "__main__":
    main()



#! /usr/bin/env python3
"""
## Command ``v``

Associative table to save small variables. See `v --help` for more details.

### Example

```
$ # save the value '+44 1234 123456' under the key 'john_number'
$ v john_number -a '+44 1234 123456'
$ # print John's number
$ v john_number
$ # forget John's number
$ v john_number -r
```
"""

import click
import sys, os
from ._json import *
from ._srl import *

@click.command()
@click.argument('keys', required = False, nargs=-1)
@decorate_srl
def main(keys, **kwargs):
    """Contains variables."""
    
    command = 'v'
    filename = resolve_json_filename(command)

    if handle_srl(command, filename, keys, **kwargs):
        return

    values = get_json(filename, keys)
    for value in values:
        print(value)

if __name__ == "__main__":
    main()



#! /usr/bin/env python3
"""
## Command ``rand``

Display a random integer. See `rand --help` for more details.

### Example

```
$ # randomly draw an integer between 5 and 10
$ rand -m 5 -M 10
```
"""

import click
import random
import sys

@click.command()
@click.option('--min', '-m', help='minimum value', default=0,)
@click.option('--max', '-M', help='maximum value', default=6,)
def main(min, max):
    """Print a random number."""
    print(random.randint(min,max))

if __name__ == "__main__":
    main()



#! /usr/bin/env python3
"""
## Command ``f``

Open your favourite files with ease. See `f --help` for more details.

### Example

```
$ cd ~
$ # associate permanently the key 'bashrc' to the file '~/.bashrc'
$ f bashrc -a .bashrc
$ cd ~/Documents/games
$ # open the '~/.bashrc' file
$ f bashrc
```
"""

import click
import sys, os
from ._json import *
from ._srl import *
import subprocess
from subprocess import DEVNULL

@click.command()
@click.argument('keys', required = False, nargs=-1)
@decorate_srl

def main(keys, **kwargs):
    """open directories"""

    command = 'f'
    filename = resolve_json_filename(command)

    if handle_srl(command, filename, keys, type='file', **kwargs):
        return

    files = get_json(filename, keys)

    cmd = 'xdg-open {}' if sys.platform in ['linux','linux2'] else 'open "{}"'
    for file in files:
        subprocess.call(cmd.format(file), shell=True, stderr=DEVNULL, stdout=DEVNULL)

if __name__ == "__main__":
    main()



#! /usr/bin/env python3
"""
    The ``._json`` module
    ======================
    Contains all the functions related to the reading and mofifications of the ``.json`` files
"""

import json
import os
import sys

PYTHON_PATH = os.path.dirname(os.path.realpath(__file__))
HOME_PATH = os.path.expanduser('~')
CONFIG_PATH = os.path.join(HOME_PATH,'.config')
DATA_PATH = os.path.join(CONFIG_PATH, 'pyqo')

def resolve_json_filename(command):
    if not os.path.isdir(CONFIG_PATH):
        os.mkdir(CONFIG_PATH)
    if not os.path.isdir(DATA_PATH):
        os.mkdir(DATA_PATH)
    #init default filename
    filename = os.path.join(DATA_PATH,'{}.json'.format(command))
    #test if config
    config = read_json(os.path.join(DATA_PATH,'config.json'))
    key_name = '{}_json'.format(command)
    if key_name in config:
            filename = config[key_name]

    return filename

def set_config(command, datafile):
    config_file = os.path.join(DATA_PATH,'config.json')
    key_name = '{}_json'.format(command)
    data = read_json(config_file)
    data[key_name] = datafile
    write_json(config_file, data)
    
def read_json(filename):
    if os.path.isfile(filename):
        with open(filename, encoding='utf-8') as f:
            data = json.loads(f.read())
    else:
        data = {}
    return data

def list_json(filename, keys):
    from ._printer import print_map
    if keys is None or len(keys)<1 or keys[0] is None:
        data = read_json(filename)
        print_map(data)
    else:
        values = get_json(filename, keys)
        print(values)
        dico = {keys[i] : values[i] for i in range(len(keys))}
        print_map(dico)

def get_json(filename, keys):
    data = read_json(filename)
    l = []
    for key in keys:
        if key in data:
            l.append(data[key])
        else:
            print('The key "{}" has no attributed value.'.format(key))
            del key
    return l

def write_json(filename, data):
    with open(filename, 'w', encoding = 'utf-8') as f:
        json.dump(data, f)

def set_json(filename, map):
    data = read_json(filename)
    for key, value in map.items():
        data[key] = value
    write_json(filename, data)

def remove_json(filename, keys):
    data = read_json(filename)
    for key in keys:
        data.pop(key)
    write_json(filename, data)



#! /usr/bin/env python3
"""
## Command ``yget``

Downloads in the current folder the youtube video whose url is passed as a parameter. See `yget --help` for more details.

### Example

```
$ # downloads the youtube video '"Sweet Victory" Performance'
$ yget https://www.youtube.com/watch?v=k9iYm9PEAHg
```
"""

import click
import pytube
import sys
import os

@click.command()
@click.argument('url')
def main(url):
    """Download the youtube video in the current directory."""

    print("Downloading {}".format(url))
    try:
        yt = pytube.YouTube(url)
        stream = yt.streams.first()
        stream.download(os.getcwd())
    except Exception as e:
        print(e)
        input("Press Enter to continue...")

if __name__ == "__main__":
    main()



#! /usr/bin/env python3
"""
    The ``_srl`` module
    ===================
    Contains all the functions related to the SRL ("set", "remove" and "list") commands.
"""

import click
from ._json import *

def decorate_srl(f):
    return click.option('--assign', '-a', help='Assign a value to the given key.', default=None,)(
    click.option('--remove', '-r', help='Remove a key.', is_flag=True,)(
    click.option('--list', '-l', help='List all the keys if no given key. Otherwise print the associated value.', is_flag=True,)(
    click.option('--set_datafile', '-s', help='Set considered datafile for this command.', default=None,)(
    f))))

def handle_srl(command, filename, keys, assign = None, remove = None, list = None, set_datafile = None, type = None):
    if assign is not None:
        if len(keys)!=1:
            print("When setting a new value, you should provide exactly one key.")
            exit()
        if type in ['dir','file']:
            assign = resolve_path(assign)
        set_json(filename, {keys[0]: assign})
        return True

    if remove:
        remove_json(filename, keys)
        return True

    if list:
        list_json(filename, keys)
        return True
        
    if set_datafile is not None:
        if len(keys)>0:
            print('Should not provide any key when setting a new datafile')
            exit()
        set_config(command, set_datafile)
        return True

    return False

def resolve_path(set):
    if os.path.exists(set):
        if not os.path.isabs(set):
            set = os.path.join(os.getcwd(),set)
    else:
        exit("Given file not found. Aborting.")
    return set.replace('\\','/')



#! /usr/bin/env python3
"""
## Command ``i``

Open your favourite websites with ease. See `i --help` for more details.

### Example

```
$ # associate permanently the key 'github' to 'http://www.github.com'
$ i github -a http://www.github.com
$ # associate permanently the key 'so' to 'https://stackoverflow.com/'
$ i so -a https://stackoverflow.com/
$ # open the two websites on the existing webbrowser window
$ i github so
$ # open github and performs a google search for 'python' on a new webbrowser window
$ i -n github -g python
```
"""

import click
import sys, os
from ._json import *
from ._srl import *
from ._webbrowser import open_new, open_new_tab
from urllib.parse import quote

@click.command()
@click.argument('keys', required = False, nargs=-1)
@click.option('--new_window', '-n', help='Open results in a new window.', is_flag=True,)
@click.option('--google', '-g', help='Perform a google search.', multiple=True)
@decorate_srl
def main(keys, new_window, google, **kwargs):
    """Open websites."""

    command = 'i'
    filename = resolve_json_filename(command)

    if handle_srl(command, filename, keys, **kwargs):
        return

    value_keys = get_json(filename, keys)

    google_url = 'https://www.google.com/search?q={}'
    value_google = [google_url.format(quote(r)) for r in google]

    urls = value_keys + value_google
    if len(keys)==0:
        urls = ['https://www.google.com']

    for i in range(len(urls)):
        if i == 0 and new_window:
            open_new(urls[i])
        else:
            open_new_tab(urls[i])

if __name__ == "__main__":
    main()



#! /usr/bin/env python3
"""
## Command ``ant``

Searches for all antonyms of the word given in parameter (french). See `ant --help` for more details.

### Example

```
$ # searches for all antonyms of 'gentil'
$ ant gentil
```
"""

import click
from bs4 import BeautifulSoup
import requests
import re
from ._printer import print_list
from urllib.parse import quote

@click.command()
@click.argument('word')
def main(word):
    """Retrieve synonyms."""
    url_antonyme = 'http://www.antonyme.org/antonyme/{}'
    url = url_antonyme.format(quote(word))

    r = requests.get(url)
    soup = BeautifulSoup(r.content, 'html.parser')
    fiche = soup.find('div', attrs={"class": "fiche"})
    table = fiche.find('ul', attrs={"class": "synos"})
    if table is None:
        print('Nothing found...')
        return
    syns = table.find_all('a', attrs={"href": re.compile(r'http://www.antonyme.org/antonyme/*')})
    syns_txt = [syn.text.replace(u'\xa0', u'') for syn in syns]

    print_list(syns_txt)

if __name__ == "__main__":
    main()



#! /usr/bin/env python3
"""
## Command ``syn``

Searches for all synonyms of the word given in parameter (french). See `syn --help` for more details.

### Example

```
$ # searches for all synonyms of 'gentil'
$ syn gentil
```
"""

import click
from bs4 import BeautifulSoup
import requests
import re
from ._printer import print_list
from urllib.parse import quote

@click.command()
@click.argument('word')
def main(word):
    """Retrieve synonyms."""
    url_crisco = 'http://www.crisco.unicaen.fr/des/synonymes/{}'
    url = url_crisco.format(word)

    r = requests.get(url)
    soup = BeautifulSoup(r.content, 'html.parser')
    table = soup.find('table')
    if table is None:
        print('Nothing found...')
        return
    syns = table.find_all('a', attrs={"href": re.compile(r'des/synonymes/*')})
    syns_txt = [syn.text.replace(u'\xa0', u'') for syn in syns]
    print_list(syns_txt)

if __name__ == "__main__":
    main()



#! /usr/bin/env python3
"""
    The ``._printer`` module
    =======================
    Contains all functions whose purpose is to display results of this library.
"""

WIDTH = 19
PER_LINE = 4

def print_list(datas):
    r = ""
    for idx,data in enumerate(sorted(datas)):
        r+="{d:<{width}}".format(d=data, width=WIDTH)
        if (idx+1)%PER_LINE == 0:
            r+="\n"
    print(r)

def print_map(datas):
    for key, value in datas.items():
        print('{key:<{width}}: {value}'.format(key=key, width=WIDTH, value=value))



#! /usr/bin/env python3
"""
## Command ``d``

Open the file manager to your favourite directories with ease.
The command `d` shares its data with the command `c`.
See `d --help` for more details.

### Example

```
$ cd ~/Documents/games
$ # open the current working directory, here '~/Documents/games'
$ d
$ # associate permanently the key 'films' to '~/Documents/films'
$ d films -a /home/pyqo/Documents/films
$ # open '~/Documents/films'
$ d films
```
"""
import click
import sys, os
from ._json import *
from ._srl import *
import subprocess
from subprocess import DEVNULL

@click.command()
@click.argument('keys', required = False, nargs=-1)
@decorate_srl

def main(keys, **kwargs):
    """Open directories."""
    command = 'd'
    filename = resolve_json_filename(command)

    if handle_srl(command, filename, keys, type='file', **kwargs):
        return

    cmd = 'xdg-open {}' if sys.platform in ['linux','linux2'] else 'open "{}"'

    if len(keys)<1:
        dirs = [os.getcwd()]
    else:
        dirs = get_json(filename, keys)

    for dir in dirs:
        subprocess.call(cmd.format(dir), shell=True, stderr=DEVNULL, stdout=DEVNULL)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import re, os
from pyqo import __version__
from pyqo import __author__
from pyqo import __email__

PYQ_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)),'pyqo')

#commands
def commands():
    files = os.listdir(PYQ_PATH)
    pattern = re.compile("^[^_].*.py$")
    commands = [file[:-3] for file in files if pattern.match(file)]
    return commands

COMMANDS = commands()
CONSOLE_SCRIPTS = ['{c}=pyqo.{c}:main'.format(c=command) for command in COMMANDS]
SCRIPTS = ['bin/c','bin/c.bat']

#requirements
def requirements():
    with open('requirements.txt','r',encoding = 'utf-8') as f:
        lines = f.readlines()
        return [line.replace('==','>=').strip() for line in lines]
REQUIREMENTS = requirements()

#readme
with open('README.md', 'r', encoding = 'utf-8') as f:
    README = '\n'.join(f.readlines())

#setup
setup(
    name='pyqo',
    version=__version__,
    description='Useful collection of command line scripts.',
    long_description_content_type='text/markdown',
    long_description=README,
    author=__author__,
    author_email=__email__,
    url='https://github.com/Whenti/pyqo',
    packages = ['pyqo'],
    package_data={'pyqo': ['data/*.json']},
    package_dir={'pyqo':'pyqo'},
    include_package_data=True,
    install_requires=REQUIREMENTS,
    license='Apache License',
    zip_safe=False,
    keywords='pip requirements imports',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
    scripts = SCRIPTS,
    entry_points={
        'console_scripts': CONSOLE_SCRIPTS,
    },
)



#! /usr/bin/env python3
"""
## Command ``say``

Launches a synthesized voice that reads the given parameters. See `say --help` for more details.

### Example

```
$ say "Hi, how are you ?"
```
"""


import click
import sys, os
import pyttsx3

@click.command()
@click.argument('speech', nargs=-1)
def main(speech):
    """Simulate the reading of the given speech."""
    engine = pyttsx3.init()
    for word in speech:
        engine.say(speech)
    engine.runAndWait()

if __name__ == "__main__":
    main()



#! /usr/bin/env python3
"""
## Command ``c``

Set the working directory of the command line to your favourite directories with ease.
For a script to alter the current environment, it requires `source`'ing in linux.
We suggest you to create an alias to avoid doing it manually : `alias c="source c"`.
The command `c` shares its data with the command `d`.
See `c --help` for more details.

### Example

```
$ cd ~/Documents/games
$ # associate permanently the key 'games' to '~/Documents/games'
$ c games -a .
$ # associate permanently the key 'films' to '~/Documents/films'
$ c films -a /home/pyqo/Documents/films
$ # equivalent to 'cd ~/Documents/films'
$ c films
$ # equivalent to 'cd ~/Documents/games'
$ c games
```
"""

import click
import sys, os
from ._json import *
from ._srl import *

@click.command()
@click.argument('keys', required = False, nargs = -1)
@decorate_srl
def main(keys, **kwargs):
    """Navigate through directories."""

    command = 'd'
    filename = resolve_json_filename(command)

    if handle_srl(command, filename, keys, type='file', **kwargs):
        exit()

    if len(keys)!=1:
        print("When changing directory, you should provide exactly one key.")
        exit()

    values = get_json(filename, keys)

    if len(values)<1:
        print("Key not known, aborting.")
        exit()
    exit(values[0])

if __name__ == "__main__":
    main()
